@@ -522,7 +522,7 @@ class AutoSEResNet(AutoModel):
             self._best_result = correct_rate
             self._best_state = deepcopy(self.model.state_dict())
             self.push_log('Validation result is better than last epoch.')
-            return False
+            return self._epoch >= self.epochs
         else:
             self._overfit_index += 1
             msg = f'Validation result gets worse for {self._overfit_index} consecutive times.'
@@ -553,6 +553,7 @@ class AutoSEResNet(AutoModel):
         self.config.label2id = dict(sorted(self.config.label2id.items()))
 
         is_finished = False
+        self._epoch = 0
         while not is_finished:
             self._epoch += 1
             self.push_log(f'Begin training of epoch {self._epoch}.')
@@ -658,7 +659,7 @@ class AutoSEResNetFedAvg(AutoSEResNet, AutoFedAvgModel):
     def fine_tuned_files_dict(self) -> Optional[Dict[str, str]]:
         param_file = os.path.join(self.result_dir, self.config.param_file)
         with open(param_file, 'wb') as f:
-            torch.save(self._best_state, f)
+            torch.save(self.scheduler.best_state_dict, f)
         config_file = os.path.join(self.result_dir, 'config.json')
         with open(config_file, 'w') as f:
             f.write(json.dumps(asdict(self.config), ensure_ascii=False))
@@ -672,8 +673,6 @@ class SEResNetFedAvgScheduler(AutoFedAvgScheduler):
 
     def __init__(self,
                  auto_proxy: AutoSEResNetFedAvg,
-                 min_clients: int,
-                 max_clients: int,
                  max_rounds: int = 0,
                  merge_epochs: int = 1,
                  calculation_timeout: int = 300,
@@ -681,8 +680,6 @@ class SEResNetFedAvgScheduler(AutoFedAvgScheduler):
                  log_rounds: int = 0,
                  involve_aggregator: bool = False):
         super().__init__(auto_proxy=auto_proxy,
-                         min_clients=min_clients,
-                         max_clients=max_clients,
                          max_rounds=max_rounds,
                          merge_epochs=merge_epochs,
                          calculation_timeout=calculation_timeout,
@@ -709,7 +706,7 @@ class SEResNetFedAvgScheduler(AutoFedAvgScheduler):
     def train_an_epoch(self):
         self.auto_proxy.train_an_epoch()
 
-    def test(self):
+    def run_test(self):
         avg_loss, correct_rate = self.auto_proxy.run_test()
 
         self.tb_writer.add_scalar('test_results/avg_loss', avg_loss, self.current_round)
@@ -739,7 +736,7 @@ class SEResNetFedAvgScheduler(AutoFedAvgScheduler):
             self._best_result = correct_rate
             self._best_state = deepcopy(self.state_dict())
             self.push_log('Validation result is better than last epoch.')
-            return False
+            return self._is_reach_max_rounds()
         else:
             self._overfit_index += 1
             msg = f'Validation result gets worse for {self._overfit_index} consecutive times.'
