@@ -2,7 +2,6 @@
 
 import os
 from time import time
-from typing import Dict
 
 import torch
 import torch.nn as nn
@@ -11,9 +10,8 @@ import torchvision
 from torch import optim
 from torch.utils.data import DataLoader
 
-from ... import logger
-from ...fed_avg import (FedAvgScheduler, FedSGDScheduler,
-                        SecureFedAvgScheduler, register_metrics)
+from ... import get_dataset_dir, logger
+from ...fed_avg import FedAvgScheduler, FedSGDScheduler, SecureFedAvgScheduler
 from ...fed_avg.dp_fed_avg import DPFedAvgScheduler
 from . import DEV_TASK_ID
 from .demo_FedIRM import DemoFedIRM
@@ -50,51 +48,41 @@ class ConvNet(nn.Module):
 class DemoAvg(FedAvgScheduler):
 
     def __init__(self,
-                 min_clients: int,
-                 max_clients: int,
-                 name: str = None,
                  max_rounds: int = 0,
-                 merge_epoch: int = 1,
+                 merge_epochs: int = 1,
                  calculation_timeout: int = 300,
                  log_rounds: int = 0,
-                 is_centralized: bool = True,
                  involve_aggregator: bool = False,
                  batch_size: int = 64,
                  learning_rate: float = 0.01,
                  momentum: float = 0.5) -> None:
-        super().__init__(min_clients=min_clients,
-                         max_clients=max_clients,
-                         name=name,
-                         max_rounds=max_rounds,
-                         merge_epochs=merge_epoch,
+        super().__init__(max_rounds=max_rounds,
+                         merge_epochs=merge_epochs,
                          calculation_timeout=calculation_timeout,
                          log_rounds=log_rounds,
-                         is_centralized=is_centralized,
                          involve_aggregator=involve_aggregator)
         self.batch_size = batch_size
         self.learning_rate = learning_rate
         self.momentum = momentum
 
-        self._time_metrics = None
-
         self.device = torch.device('cuda' if torch.cuda.is_available() else 'cpu')
         self.seed = 42
         torch.manual_seed(self.seed)
 
-    def make_model(self) -> nn.Module:
+    def build_model(self) -> nn.Module:
         model = ConvNet()
-        return model
+        return model.to(self.device)
 
-    def make_optimizer(self) -> optim.Optimizer:
+    def build_optimizer(self, model: nn.Module) -> optim.Optimizer:
         assert self.model, 'must initialize model first'
-        return optim.SGD(self.model.parameters(),
+        return optim.SGD(model.parameters(),
                          lr=self.learning_rate,
                          momentum=self.momentum)
 
-    def make_train_dataloader(self) -> DataLoader:
+    def build_train_dataloader(self) -> DataLoader:
         return DataLoader(
             torchvision.datasets.MNIST(
-                os.path.join(self.name, 'data'),
+                get_dataset_dir(self.task_id),
                 train=True,
                 download=True,
                 transform=torchvision.transforms.Compose([
@@ -106,10 +94,10 @@ class DemoAvg(FedAvgScheduler):
             shuffle=True
         )
 
-    def make_test_dataloader(self) -> DataLoader:
+    def build_test_dataloader(self) -> DataLoader:
         return DataLoader(
             torchvision.datasets.MNIST(
-                os.path.join(self.name, 'data'),
+                get_dataset_dir(self.task_id),
                 train=False,
                 download=True,
                 transform=torchvision.transforms.Compose([
@@ -121,25 +109,16 @@ class DemoAvg(FedAvgScheduler):
             shuffle=False
         )
 
-    def state_dict(self) -> Dict[str, torch.Tensor]:
-        return self.model.state_dict()
-
-    def load_state_dict(self, state_dict: Dict[str, torch.Tensor]):
-        self.model.load_state_dict(state_dict)
-
     def validate_context(self):
         super().validate_context()
-        train_loader = self.make_train_dataloader()
-        assert train_loader and len(train_loader) > 0, 'failed to load train data'
-        logger.info(f'There are {len(train_loader.dataset)} samples for training.')
-        test_loader = self.make_test_dataloader()
-        assert test_loader and len(test_loader) > 0, 'failed to load test data'
-        logger.info(f'There are {len(test_loader.dataset)} samples for testing.')
+        assert self.train_loader and len(self.train_loader) > 0, 'failed to load train data'
+        logger.info(f'There are {len(self.train_loader.dataset)} samples for training.')
+        assert self.test_loader and len(self.test_loader) > 0, 'failed to load test data'
+        logger.info(f'There are {len(self.test_loader.dataset)} samples for testing.')
 
-    def train(self) -> None:
+    def train_an_epoch(self) -> None:
         self.model.train()
-        train_loader = self.make_train_dataloader()
-        for data, labels in train_loader:
+        for data, labels in self.train_loader:
             data: torch.Tensor
             labels: torch.Tensor
             data, labels = data.to(self.device), labels.to(self.device)
@@ -149,16 +128,13 @@ class DemoAvg(FedAvgScheduler):
             loss.backward()
             self.optimizer.step()
 
-    @register_metrics(name='timer', keys=['run_time'])
-    @register_metrics(name='test_results', keys=['average_loss', 'accuracy', 'correct_rate'])
-    def test(self):
+    def run_test(self):
         start = time()
         self.model.eval()
         test_loss = 0
         correct = 0
         with torch.no_grad():
-            test_loader = self.make_test_dataloader()
-            for data, labels in test_loader:
+            for data, labels in self.test_loader:
                 data: torch.Tensor
                 labels: torch.Tensor
                 data, labels = data.to(self.device), labels.to(self.device)
@@ -167,8 +143,8 @@ class DemoAvg(FedAvgScheduler):
                 pred = output.max(1, keepdim=True)[1]
                 correct += pred.eq(labels.view_as(pred)).sum().item()
 
-        test_loss /= len(test_loader.dataset)
-        accuracy = correct / len(test_loader.dataset)
+        test_loss /= len(self.test_loader.dataset)
+        accuracy = correct / len(self.test_loader.dataset)
         correct_rate = 100. * accuracy
         logger.info(f'Test set: Average loss: {test_loss:.4f}')
         logger.info(
@@ -176,53 +152,44 @@ class DemoAvg(FedAvgScheduler):
         )
 
         end = time()
-        self.get_metrics('timer').append_metrics_item({'run_time': end - start})
-        self.get_metrics('test_results').append_metrics_item({
-            'average_loss': test_loss,
-            'accuracy': accuracy,
-            'correct_rate': correct_rate
-        })
+
+        self.tb_writer.add_scalar('timer/run_time', end - start, self.current_round)
+        self.tb_writer.add_scalar('test_results/average_loss', test_loss, self.current_round)
+        self.tb_writer.add_scalar('test_results/accuracy', accuracy, self.current_round)
+        self.tb_writer.add_scalar('test_results/correct_rate', correct_rate, self.current_round)
 
 
 class DemoSGD(FedSGDScheduler):
 
     def __init__(self,
-                 min_clients: int,
-                 name: str = None,
                  max_rounds: int = 0,
                  calculation_timeout: int = 300,
                  log_rounds: int = 0,
-                 is_centralized: bool = True,
                  learning_rate: float = 0.01,
                  momentum: float = 0.5) -> None:
-        super().__init__(min_clients=min_clients,
-                         name=name,
-                         max_rounds=max_rounds,
+        super().__init__(max_rounds=max_rounds,
                          calculation_timeout=calculation_timeout,
-                         log_rounds=log_rounds,
-                         is_centralized=is_centralized)
+                         log_rounds=log_rounds)
         self.learning_rate = learning_rate
         self.momentum = momentum
 
-        self._time_metrics = None
-
         self.device = torch.device('cuda' if torch.cuda.is_available() else 'cpu')
         self.seed = 42
         torch.manual_seed(self.seed)
 
-    def make_model(self) -> nn.Module:
+    def build_model(self) -> nn.Module:
         model = ConvNet()
-        return model
+        return model.to(self.device)
 
-    def make_optimizer(self) -> optim.Optimizer:
+    def build_optimizer(self, model: nn.Module) -> optim.Optimizer:
         assert self.model, 'must initialize model first'
-        return optim.SGD(self.model.parameters(),
+        return optim.SGD(model.parameters(),
                          lr=self.learning_rate,
                          momentum=self.momentum)
 
-    def make_train_dataloader(self) -> DataLoader:
+    def build_train_dataloader(self) -> DataLoader:
         dataset = torchvision.datasets.MNIST(
-            os.path.join(self.name, 'data'),
+            get_dataset_dir(self.task_id),
             train=True,
             download=True,
             transform=torchvision.transforms.Compose([
@@ -232,10 +199,10 @@ class DemoSGD(FedSGDScheduler):
         )
         return DataLoader(dataset=dataset, batch_size=len(dataset), shuffle=True)
 
-    def make_test_dataloader(self) -> DataLoader:
+    def build_test_dataloader(self) -> DataLoader:
         return DataLoader(
             torchvision.datasets.MNIST(
-                os.path.join(self.name, 'data'),
+                get_dataset_dir(self.task_id),
                 train=False,
                 download=True,
                 transform=torchvision.transforms.Compose([
@@ -247,25 +214,16 @@ class DemoSGD(FedSGDScheduler):
             shuffle=False
         )
 
-    def state_dict(self) -> Dict[str, torch.Tensor]:
-        return self.model.state_dict()
-
-    def load_state_dict(self, state_dict: Dict[str, torch.Tensor]):
-        self.model.load_state_dict(state_dict)
-
     def validate_context(self):
         super().validate_context()
-        train_loader = self.make_train_dataloader()
-        assert train_loader and len(train_loader) > 0, 'failed to load train data'
-        self.push_log(f'There are {len(train_loader.dataset)} samples for training.')
-        test_loader = self.make_test_dataloader()
-        assert test_loader and len(test_loader) > 0, 'failed to load test data'
-        self.push_log(f'There are {len(test_loader.dataset)} samples for testing.')
+        assert self.train_loader and len(self.train_loader) > 0, 'failed to load train data'
+        self.push_log(f'There are {len(self.train_loader.dataset)} samples for training.')
+        assert self.test_loader and len(self.test_loader) > 0, 'failed to load test data'
+        self.push_log(f'There are {len(self.test_loader.dataset)} samples for testing.')
 
-    def train(self) -> None:
+    def train_an_epoch(self) -> None:
         self.model.train()
-        train_loader = self.make_train_dataloader()
-        for data, labels in train_loader:
+        for data, labels in self.train_loader:
             data: torch.Tensor
             labels: torch.Tensor
             data, labels = data.to(self.device), labels.to(self.device)
@@ -275,15 +233,13 @@ class DemoSGD(FedSGDScheduler):
             loss.backward()
             self.optimizer.step()
 
-    @register_metrics(name='timer', keys=['run_time'])
-    @register_metrics(name='test_results', keys=['average_loss', 'accuracy', 'correct_rate'])
-    def test(self):
+    def run_test(self):
         start = time()
         self.model.eval()
         test_loss = 0
         correct = 0
         with torch.no_grad():
-            test_loader = self.make_test_dataloader()
+            test_loader = self.build_test_dataloader()
             for data, labels in test_loader:
                 data: torch.Tensor
                 labels: torch.Tensor
@@ -302,58 +258,47 @@ class DemoSGD(FedSGDScheduler):
         )
 
         end = time()
-        self.get_metrics('timer').append_metrics_item({'run_time': end - start})
-        self.get_metrics('test_results').append_metrics_item({
-            'average_loss': test_loss,
-            'accuracy': accuracy,
-            'correct_rate': correct_rate
-        })
+
+        self.tb_writer.add_scalar('timer/run_time', end - start, self.current_round)
+        self.tb_writer.add_scalar('test_results/average_loss', test_loss, self.current_round)
+        self.tb_writer.add_scalar('test_results/accuracy', accuracy, self.current_round)
+        self.tb_writer.add_scalar('test_results/correct_rate', correct_rate, self.current_round)
 
 
 class DemoSecure(SecureFedAvgScheduler):
 
     def __init__(self,
-                 min_clients: int,
-                 max_clients: int,
                  t: int,
-                 name: str = None,
                  max_rounds: int = 0,
-                 merge_epoch: int = 1,
+                 merge_epochs: int = 1,
                  calculation_timeout: int = 300,
-                 log_rounds: int = 0,
-                 is_centralized: bool = True) -> None:
-        super().__init__(min_clients=min_clients,
-                         max_clients=max_clients,
-                         t=t,
-                         name=name,
+                 log_rounds: int = 0) -> None:
+        super().__init__(t=t,
                          max_rounds=max_rounds,
-                         merge_epochs=merge_epoch,
+                         merge_epochs=merge_epochs,
                          calculation_timeout=calculation_timeout,
-                         log_rounds=log_rounds,
-                         is_centralized=is_centralized)
+                         log_rounds=log_rounds)
         self.batch_size = 64
         self.learning_rate = 0.01
         self.momentum = 0.5
-        self.log_interval = 5
         self.random_seed = 42
 
         torch.manual_seed(self.random_seed)
         self.device = torch.device('cuda' if torch.cuda.is_available() else 'cpu')
 
-    def make_model(self) -> nn.Module:
+    def build_model(self) -> nn.Module:
         model = ConvNet()
-        return model
+        return model.to(self.device)
 
-    def make_optimizer(self) -> optim.Optimizer:
-        assert self.model, 'must initialize model first'
-        return optim.SGD(self.model.parameters(),
+    def build_optimizer(self, model: nn.Module) -> optim.Optimizer:
+        return optim.SGD(model.parameters(),
                          lr=self.learning_rate,
                          momentum=self.momentum)
 
-    def make_train_dataloader(self) -> DataLoader:
+    def build_train_dataloader(self) -> DataLoader:
         return DataLoader(
             torchvision.datasets.MNIST(
-                os.path.join(self.name, 'data'),
+                get_dataset_dir(self.task_id),
                 train=True,
                 download=True,
                 transform=torchvision.transforms.Compose([
@@ -365,10 +310,10 @@ class DemoSecure(SecureFedAvgScheduler):
             shuffle=True
         )
 
-    def make_test_dataloader(self) -> DataLoader:
+    def build_test_dataloader(self) -> DataLoader:
         return DataLoader(
             torchvision.datasets.MNIST(
-                os.path.join(self.name, 'data'),
+                get_dataset_dir(self.task_id),
                 train=False,
                 download=True,
                 transform=torchvision.transforms.Compose([
@@ -380,25 +325,16 @@ class DemoSecure(SecureFedAvgScheduler):
             shuffle=False
         )
 
-    def state_dict(self) -> Dict[str, torch.Tensor]:
-        return self.model.state_dict()
-
-    def load_state_dict(self, state_dict: Dict[str, torch.Tensor]):
-        self.model.load_state_dict(state_dict)
-
     def validate_context(self):
         super().validate_context()
-        train_loader = self.make_train_dataloader()
-        assert train_loader and len(train_loader) > 0, 'failed to load train data'
-        self.push_log(f'There are {len(train_loader.dataset)} samples for training.')
-        test_loader = self.make_test_dataloader()
-        assert test_loader and len(test_loader) > 0, 'failed to load test data'
-        self.push_log(f'There are {len(test_loader.dataset)} samples for testing.')
+        assert self.train_loader and len(self.train_loader) > 0, 'failed to load train data'
+        self.push_log(f'There are {len(self.train_loader.dataset)} samples for training.')
+        assert self.test_loader and len(self.test_loader) > 0, 'failed to load test data'
+        self.push_log(f'There are {len(self.test_loader.dataset)} samples for testing.')
 
-    def train(self) -> None:
+    def train_an_epoch(self) -> None:
         self.model.train()
-        train_loader = self.make_train_dataloader()
-        for data, labels in train_loader:
+        for data, labels in self.train_loader:
             data: torch.Tensor
             labels: torch.Tensor
             data, labels = data.to(self.device), labels.to(self.device)
@@ -408,13 +344,12 @@ class DemoSecure(SecureFedAvgScheduler):
             loss.backward()
             self.optimizer.step()
 
-    @register_metrics(name='test_results', keys=['average_loss', 'correct_rate'])
-    def test(self):
+    def run_test(self):
         self.model.eval()
         test_loss = 0
         correct = 0
         with torch.no_grad():
-            test_loader = self.make_test_dataloader()
+            test_loader = self.build_test_dataloader()
             for data, labels in test_loader:
                 data: torch.Tensor
                 labels: torch.Tensor
@@ -431,62 +366,52 @@ class DemoSecure(SecureFedAvgScheduler):
             f'Test set: Accuracy: {correct}/{len(test_loader.dataset)} ({correct_rate:.2f}%)'
         )
 
-        self.get_metrics('test_results').append_metrics_item({
-            'average_loss': test_loss,
-            'correct_rate': correct_rate
-        })
+        self.tb_writer.add_scalar('test_results/average_loss', test_loss, self.current_round)
+        self.tb_writer.add_scalar('test_results/correct_rate', correct_rate, self.current_round)
 
 
 class DemoDP(DPFedAvgScheduler):
 
     def __init__(self,
-                 min_clients: int,
                  w_cap: int,
                  q: float,
                  S: float,
                  z: float,
-                 name: str = None,
                  max_rounds: int = 0,
-                 merge_epoch: int = 1,
+                 merge_epochs: int = 1,
                  calculation_timeout: int = 300,
                  log_rounds: int = 0,
-                 is_centralized: bool = True,
                  involve_aggregator: bool = False) -> None:
-        super().__init__(min_clients=min_clients,
-                         w_cap=w_cap,
+        super().__init__(w_cap=w_cap,
                          q=q,
                          S=S,
                          z=z,
-                         name=name,
                          max_rounds=max_rounds,
-                         merge_epochs=merge_epoch,
+                         merge_epochs=merge_epochs,
                          calculation_timeout=calculation_timeout,
                          log_rounds=log_rounds,
-                         is_centralized=is_centralized,
                          involve_aggregator=involve_aggregator)
         self.batch_size = 64
         self.learning_rate = 0.01
         self.momentum = 0.5
-        self.log_interval = 5
         self.random_seed = 42
 
         torch.manual_seed(self.random_seed)
         self.device = torch.device('cuda' if torch.cuda.is_available() else 'cpu')
 
-    def make_model(self) -> nn.Module:
+    def build_model(self) -> nn.Module:
         model = ConvNet()
-        return model
+        return model.to(self.device)
 
-    def make_optimizer(self) -> optim.Optimizer:
-        assert self.model, 'must initialize model first'
-        return optim.SGD(self.model.parameters(),
+    def build_optimizer(self, model: nn.Module) -> optim.Optimizer:
+        return optim.SGD(model.parameters(),
                          lr=self.learning_rate,
                          momentum=self.momentum)
 
-    def make_train_dataloader(self) -> DataLoader:
+    def build_train_dataloader(self) -> DataLoader:
         return DataLoader(
             torchvision.datasets.MNIST(
-                os.path.join(self.name, 'data'),
+                get_dataset_dir(self.task_id),
                 train=True,
                 download=True,
                 transform=torchvision.transforms.Compose([
@@ -498,10 +423,10 @@ class DemoDP(DPFedAvgScheduler):
             shuffle=True
         )
 
-    def make_test_dataloader(self) -> DataLoader:
+    def build_test_dataloader(self) -> DataLoader:
         return DataLoader(
             torchvision.datasets.MNIST(
-                os.path.join(self.name, 'data'),
+                get_dataset_dir(self.task_id),
                 train=False,
                 download=True,
                 transform=torchvision.transforms.Compose([
@@ -513,20 +438,12 @@ class DemoDP(DPFedAvgScheduler):
             shuffle=False
         )
 
-    def state_dict(self) -> Dict[str, torch.Tensor]:
-        return self.model.state_dict()
-
-    def load_state_dict(self, state_dict: Dict[str, torch.Tensor]):
-        self.model.load_state_dict(state_dict)
-
     def validate_context(self):
         super().validate_context()
-        train_loader = self.make_train_dataloader()
-        assert train_loader and len(train_loader) > 0, 'failed to load train data'
-        self.push_log(f'There are {len(train_loader.dataset)} samples for training.')
-        test_loader = self.make_test_dataloader()
-        assert test_loader and len(test_loader) > 0, 'failed to load test data'
-        self.push_log(f'There are {len(test_loader.dataset)} samples for testing.')
+        assert self.train_loader and len(self.train_loader) > 0, 'failed to load train data'
+        self.push_log(f'There are {len(self.train_loader.dataset)} samples for training.')
+        assert self.test_loader and len(self.test_loader) > 0, 'failed to load test data'
+        self.push_log(f'There are {len(self.test_loader.dataset)} samples for testing.')
 
     def train_a_batch(self, *batch_train_data):
         data: torch.Tensor
@@ -539,13 +456,12 @@ class DemoDP(DPFedAvgScheduler):
         loss.backward()
         self.optimizer.step()
 
-    @register_metrics(name='test_results', keys=['average_loss', 'correct_rate'])
-    def test(self):
+    def run_test(self):
         self.model.eval()
         test_loss = 0
         correct = 0
         with torch.no_grad():
-            test_loader = self.make_test_dataloader()
+            test_loader = self.build_test_dataloader()
             for data, labels in test_loader:
                 data: torch.Tensor
                 labels: torch.Tensor
@@ -562,10 +478,8 @@ class DemoDP(DPFedAvgScheduler):
             f'Test set: Accuracy: {correct}/{len(test_loader.dataset)} ({correct_rate:.2f}%)'
         )
 
-        self.get_metrics('test_results').append_metrics_item({
-            'average_loss': test_loss,
-            'correct_rate': correct_rate
-        })
+        self.tb_writer.add_scalar('test_results/average_loss', test_loss, self.current_round)
+        self.tb_writer.add_scalar('test_results/correct_rate', correct_rate, self.current_round)
 
 
 def get_task_id() -> str:
@@ -582,54 +496,42 @@ def get_scheduler(mode: str = VANILLA) -> FedAvgScheduler:
         os.remove(pickle_file)
 
     if mode == VANILLA:
-        scheduler = DemoAvg(min_clients=3,
-                            max_clients=3,
-                            name='demo_fed_avg',
-                            max_rounds=5,
+        scheduler = DemoAvg(max_rounds=5,
                             log_rounds=1,
                             calculation_timeout=60,
                             involve_aggregator=True)
 
     elif mode == SGD:
-        scheduler = DemoSGD(min_clients=3,
-                            name='demo_fed_sgd',
-                            max_rounds=50,
+        scheduler = DemoSGD(max_rounds=200,
                             log_rounds=1,
                             calculation_timeout=60)
 
     elif mode == DP:
-        scheduler = DemoDP(min_clients=2,
-                           w_cap=20000,
+        scheduler = DemoDP(w_cap=20000,
                            q=0.9,
                            S=1,
                            z=0.1,
-                           name='demo_dp_fed_avg',
-                           max_rounds=5,
+                           max_rounds=10,
                            log_rounds=1,
                            calculation_timeout=60,
                            involve_aggregator=True)
 
     elif mode == SECURE:
-        scheduler = DemoSecure(min_clients=3,
-                               max_clients=3,
-                               t=2,
-                               name='demo_secure_avg',
+        scheduler = DemoSecure(t=2,
                                max_rounds=5,
                                log_rounds=1,
                                calculation_timeout=120)
 
     elif mode == FED_IRM:
+        root_dir = '/data/alphamed/alphamed-dataset/tutorials/FedIRM/'
         scheduler = DemoFedIRM(
-            min_clients=3,
-            max_clients=3,
-            root_path=os.path.join(CURRENT_DIR, 'gtr21/ISIN-2018/train_image_224'),
-            csv_file_train=os.path.join(CURRENT_DIR, 'train.csv'),
-            csv_file_test=os.path.join(CURRENT_DIR, 'test.csv'),
-            name='demo_Fed_IRM',
+            root_path=os.path.join(root_dir, 'gtr21/ISIN-2018/train_image_224'),
+            csv_file_train=os.path.join(root_dir, 'train.csv'),
+            csv_file_test=os.path.join(root_dir, 'test.csv'),
             max_rounds=50,
             log_rounds=5,
             calculation_timeout=3600,
-            data_channel_timeout=(600, 200)
+            involve_aggregator=True
         )
 
     with open(pickle_file, 'w+b') as pf:
