@@ -1,6 +1,7 @@
 """HeteroNN demos."""
 
 import os
+from hashlib import md5
 from time import time
 from typing import Dict, List, Set, Tuple, Union
 
@@ -9,15 +10,13 @@ import torch
 import torch.nn as nn
 import torch.nn.functional as F
 import torch.optim as optim
-import torch.utils.data as td
 import torchvision
+from torch.utils.data import DataLoader, Subset
 
 from ... import logger
 from ...hetero_nn import (HeteroNNCollaboratorScheduler, HeteroNNHostScheduler,
                           SecureHeteroNNCollaboratorScheduler,
                           SecureHeteroNNHostScheduler)
-from ...hetero_nn.psi import (RSAPSICollaboratorScheduler,
-                              RSAPSIInitiatorScheduler)
 from . import DEV_TASK_ID
 
 CURRENT_DIR = os.path.dirname(os.path.abspath(__file__))
@@ -37,7 +36,7 @@ class ConvNet(nn.Module):
         self.conv2 = nn.Conv2d(in_channels=10, out_channels=20, kernel_size=5)
         self.conv2_drop = nn.Dropout2d()
         self.fc1 = nn.Linear(in_features=80, out_features=50)
-        self.fc2 = nn.Linear(in_features=50, out_features=10)
+        self.fc2 = nn.Linear(in_features=50, out_features=20)
 
     def forward(self, x):
         x = F.relu(F.max_pool2d(self.conv1(x), 2))
@@ -52,10 +51,12 @@ class InferModule(nn.Module):
 
     def __init__(self) -> None:
         super().__init__()
-        self.linear = nn.Linear(20, 10)
+        self.fc1 = nn.Linear(40, 20)
+        self.fc2 = nn.Linear(20, 10)
 
     def forward(self, input):
-        out = self.linear(input)
+        out = F.relu(self.fc1(input))
+        out = self.fc2(out)
         return F.log_softmax(out, dim=-1)
 
 
@@ -65,57 +66,33 @@ class DemoHeteroHost(HeteroNNHostScheduler):
                  feature_key: str,
                  batch_size: int,
                  data_dir: str,
-                 name: str = None,
                  max_rounds: int = 0,
                  calculation_timeout: int = 300,
                  schedule_timeout: int = 30,
-                 log_rounds: int = 0,
-                 is_feature_trainable: bool = True) -> None:
+                 log_rounds: int = 0) -> None:
         super().__init__(feature_key=feature_key,
-                         name=name,
                          max_rounds=max_rounds,
                          calculation_timeout=calculation_timeout,
                          schedule_timeout=schedule_timeout,
-                         log_rounds=log_rounds,
-                         is_feature_trainable=is_feature_trainable)
+                         log_rounds=log_rounds)
         self.batch_size = batch_size
         self.data_dir = data_dir
 
-        train_dataset = torchvision.datasets.MNIST(
-            self.data_dir,
-            train=True,
-            download=True,
-            transform=torchvision.transforms.Compose([
-                torchvision.transforms.ToTensor(),
-                torchvision.transforms.Normalize((0.1307,), (0.3081,))
-            ])
-        )
-        self.train_loader = td.DataLoader(train_dataset,
-                                          batch_size=self.batch_size,
-                                          shuffle=False)
-        test_dataset = torchvision.datasets.MNIST(
-            self.data_dir,
-            train=False,
-            download=True,
-            transform=torchvision.transforms.Compose([
-                torchvision.transforms.ToTensor(),
-                torchvision.transforms.Normalize((0.1307,), (0.3081,))
-            ])
-        )
-        self.test_loader = td.DataLoader(test_dataset,
-                                         batch_size=self.batch_size,
-                                         shuffle=False)
-
     def load_local_ids(self) -> List[str]:
-        train_ids = [str(_id) for _id in self.train_loader.sampler]
-        test_ids = [str(-1 - _id) for _id in self.test_loader.sampler]
+        train_ids = [str(i) for i in range(0, 20000)]
+        test_ids = [str(i) for i in range(100000, 105000)]
         return train_ids + test_ids
 
     def split_dataset(self, id_intersection: Set[str]) -> Tuple[Set[str], Set[str]]:
-        ids = set(int(_id) for _id in id_intersection)
-        train_ids = set(str(_id) for _id in ids if _id >= 0)
-        test_ids = set(str(_id + 1) for _id in ids if _id < 0)
-        return train_ids, test_ids
+        ids = [int(_id) for _id in id_intersection]
+        ids.sort()
+        train_ids = ids[:10000]
+        test_ids = ids[10000:]
+
+        logger.info(f'Got {len(train_ids)} intersecting samples for training.')
+        logger.info(f'Got {len(test_ids)} intersecting samples for testing.')
+
+        return set(train_ids), set(test_ids)
 
     def make_feature_model(self) -> nn.Module:
         return ConvNet()
@@ -128,16 +105,46 @@ class DemoHeteroHost(HeteroNNHostScheduler):
 
     def iterate_train_feature(self,
                               feature_model: nn.Module,
-                              train_ids: List[str]) -> Tuple[torch.Tensor, torch.Tensor]:
-        assert len(train_ids) == 60000, 'Some train samples lost.'
+                              train_ids: Set[str]) -> Tuple[torch.Tensor, torch.Tensor]:
+        train_dataset = torchvision.datasets.MNIST(
+            self.data_dir,
+            train=True,
+            download=True,
+            transform=torchvision.transforms.Compose([
+                torchvision.transforms.ToTensor(),
+                torchvision.transforms.Normalize((0.1307,), (0.3081,))
+            ])
+        )
+        train_ids: List = list(train_ids)
+        train_ids.sort(key=lambda x: md5(bytes(x + self.current_round)).digest())
+
+        train_dataset = Subset(train_dataset, train_ids)
+        self.train_loader = DataLoader(train_dataset,
+                                       batch_size=self.batch_size,
+                                       shuffle=False)
+
         for _data, _labels in self.train_loader:
             _data = self._erase_right(_data)
             yield feature_model(_data), _labels
 
     def iterate_test_feature(self,
                              feature_model: nn.Module,
-                             test_ids: List[str]) -> Tuple[torch.Tensor, torch.Tensor]:
-        assert len(test_ids) == 10000, 'Some test samples lost.'
+                             test_ids: Set[str]) -> Tuple[torch.Tensor, torch.Tensor]:
+        test_dataset = torchvision.datasets.MNIST(
+            self.data_dir,
+            train=False,
+            download=True,
+            transform=torchvision.transforms.Compose([
+                torchvision.transforms.ToTensor(),
+                torchvision.transforms.Normalize((0.1307,), (0.3081,))
+            ])
+        )
+        test_ids = [(i - 100000) for i in test_ids]
+        test_dataset = Subset(test_dataset, test_ids)
+        self.test_loader = DataLoader(test_dataset,
+                                      batch_size=self.batch_size,
+                                      shuffle=False)
+
         for _data, _labels in self.test_loader:
             _data = self._erase_right(_data)
             yield feature_model(_data), _labels
@@ -186,26 +193,6 @@ class DemoHeteroHost(HeteroNNHostScheduler):
         self.tb_writer.add_scalar('test_results/accuracy', accuracy, self.current_round)
         self.tb_writer.add_scalar('test_results/correct_rate', correct_rate, self.current_round)
 
-    def validate_context(self):
-        super().validate_context()
-        assert self.train_loader and len(self.train_loader) > 0, 'failed to load train data'
-        logger.info(f'There are {len(self.train_loader.dataset)} samples for training.')
-        assert self.test_loader and len(self.test_loader) > 0, 'failed to load test data'
-        logger.info(f'There are {len(self.test_loader.dataset)} samples for testing.')
-
-    # replace data channel ports used for debuging
-    def _make_id_intersection(self) -> List[str]:
-        """Make PSI and get id intersection for training."""
-        local_ids = self.load_local_ids()
-        psi_scheduler = RSAPSIInitiatorScheduler(
-            task_id=self.task_id,
-            initiator_id=self.id,
-            ids=local_ids,
-            collaborator_ids=self._partners,
-            contractor=self.contractor
-        )
-        self._id_intersection = psi_scheduler.make_intersection()
-
 
 class DemoHeteroCollaborator(HeteroNNCollaboratorScheduler):
 
@@ -213,53 +200,29 @@ class DemoHeteroCollaborator(HeteroNNCollaboratorScheduler):
                  feature_key: str,
                  batch_size: int,
                  data_dir: str,
-                 name: str = None,
                  schedule_timeout: int = 30,
                  is_feature_trainable: bool = True) -> None:
         super().__init__(feature_key=feature_key,
-                         name=name,
                          schedule_timeout=schedule_timeout,
                          is_feature_trainable=is_feature_trainable)
         self.batch_size = batch_size
         self.data_dir = data_dir
 
-        self.train_loader = td.DataLoader(
-            torchvision.datasets.MNIST(
-                self.data_dir,
-                train=True,
-                download=True,
-                transform=torchvision.transforms.Compose([
-                    torchvision.transforms.ToTensor(),
-                    torchvision.transforms.Normalize((0.1307,), (0.3081,))
-                ])
-            ),
-            batch_size=self.batch_size,
-            shuffle=False
-        )
-        self.test_loader = td.DataLoader(
-            torchvision.datasets.MNIST(
-                self.data_dir,
-                train=False,
-                download=True,
-                transform=torchvision.transforms.Compose([
-                    torchvision.transforms.ToTensor(),
-                    torchvision.transforms.Normalize((0.1307,), (0.3081,))
-                ])
-            ),
-            batch_size=self.batch_size,
-            shuffle=False
-        )
-
     def load_local_ids(self) -> List[str]:
-        train_ids = [str(_id) for _id in self.train_loader.sampler]
-        test_ids = [str(-1 - _id) for _id in self.test_loader.sampler]
+        train_ids = [str(i) for i in range(10000, 30000)]
+        test_ids = [str(i) for i in range(103000, 107000)]
         return train_ids + test_ids
 
     def split_dataset(self, id_intersection: Set[str]) -> Tuple[Set[str], Set[str]]:
-        ids = set(int(_id) for _id in id_intersection)
-        train_ids = set(str(_id) for _id in ids if _id >= 0)
-        test_ids = set(str(_id + 1) for _id in ids if _id < 0)
-        return train_ids, test_ids
+        ids = [int(_id) for _id in id_intersection]
+        ids.sort()
+        train_ids = ids[:10000]
+        test_ids = ids[10000:]
+
+        logger.info(f'Got {len(train_ids)} intersecting samples for training.')
+        logger.info(f'Got {len(test_ids)} intersecting samples for testing.')
+
+        return set(train_ids), set(test_ids)
 
     def make_feature_model(self) -> nn.Module:
         return ConvNet()
@@ -272,39 +235,50 @@ class DemoHeteroCollaborator(HeteroNNCollaboratorScheduler):
 
     def iterate_train_feature(self,
                               feature_model: nn.Module,
-                              train_ids: List[str]) -> torch.Tensor:
-        assert len(train_ids) == 60000, 'Some train samples lost.'
+                              train_ids: Set[str]) -> torch.Tensor:
+        train_dataset = torchvision.datasets.MNIST(
+            self.data_dir,
+            train=True,
+            download=True,
+            transform=torchvision.transforms.Compose([
+                torchvision.transforms.ToTensor(),
+                torchvision.transforms.Normalize((0.1307,), (0.3081,))
+            ])
+        )
+        train_ids: List = list(train_ids)
+        train_ids.sort(key=lambda x: md5(bytes(x + self.current_round)).digest())
+
+        train_dataset = Subset(train_dataset, train_ids)
+        self.train_loader = DataLoader(train_dataset,
+                                       batch_size=self.batch_size,
+                                       shuffle=False)
+
         for _data, _ in self.train_loader:
             _data = self._erase_left(_data)
             yield feature_model(_data)
 
     def iterate_test_feature(self,
                              feature_model: nn.Module,
-                             test_ids: List[str]) -> torch.Tensor:
-        assert len(test_ids) == 10000, 'Some test samples lost.'
+                             test_ids: Set[str]) -> torch.Tensor:
+        test_dataset = torchvision.datasets.MNIST(
+            self.data_dir,
+            train=False,
+            download=True,
+            transform=torchvision.transforms.Compose([
+                torchvision.transforms.ToTensor(),
+                torchvision.transforms.Normalize((0.1307,), (0.3081,))
+            ])
+        )
+        test_ids = [(i - 100000) for i in test_ids]
+        test_dataset = Subset(test_dataset, test_ids)
+        self.test_loader = DataLoader(test_dataset,
+                                      batch_size=self.batch_size,
+                                      shuffle=False)
+
         for _data, _ in self.test_loader:
             _data = self._erase_left(_data)
             yield feature_model(_data)
 
-    def validate_context(self):
-        super().validate_context()
-        assert self.train_loader and len(self.train_loader) > 0, 'failed to load train data'
-        logger.info(f'There are {len(self.train_loader.dataset)} samples for training.')
-        assert self.test_loader and len(self.test_loader) > 0, 'failed to load test data'
-        logger.info(f'There are {len(self.test_loader.dataset)} samples for testing.')
-
-    # replace data channel ports used for debuging
-    def _make_id_intersection(self) -> List[str]:
-        """Make PSI and get id intersection for training."""
-        local_ids = self.load_local_ids()
-        psi_scheduler = RSAPSICollaboratorScheduler(
-            task_id=self.task_id,
-            collaborator_id=self.id,
-            ids=local_ids,
-            contractor=self.contractor
-        )
-        self._id_intersection = psi_scheduler.collaborate_intersection()
-
 
 class DemoSecureHeteroHost(SecureHeteroNNHostScheduler):
 
@@ -314,7 +288,6 @@ class DemoSecureHeteroHost(SecureHeteroNNHostScheduler):
                  project_layer_lr: float,
                  batch_size: int,
                  data_dir: str,
-                 name: str = None,
                  max_rounds: int = 0,
                  calculation_timeout: int = 300,
                  schedule_timeout: int = 30,
@@ -323,7 +296,6 @@ class DemoSecureHeteroHost(SecureHeteroNNHostScheduler):
         super().__init__(feature_key=feature_key,
                          project_layer_config=project_layer_config,
                          project_layer_lr=project_layer_lr,
-                         name=name,
                          max_rounds=max_rounds,
                          calculation_timeout=calculation_timeout,
                          schedule_timeout=schedule_timeout,
@@ -332,44 +304,21 @@ class DemoSecureHeteroHost(SecureHeteroNNHostScheduler):
         self.batch_size = batch_size
         self.data_dir = data_dir
 
-        self.train_loader = td.DataLoader(
-            torchvision.datasets.MNIST(
-                self.data_dir,
-                train=True,
-                download=True,
-                transform=torchvision.transforms.Compose([
-                    torchvision.transforms.ToTensor(),
-                    torchvision.transforms.Normalize((0.1307,), (0.3081,))
-                ])
-            ),
-            batch_size=self.batch_size,
-            shuffle=False
-        )
-        self.test_loader = td.DataLoader(
-            torchvision.datasets.MNIST(
-                self.data_dir,
-                train=False,
-                download=True,
-                transform=torchvision.transforms.Compose([
-                    torchvision.transforms.ToTensor(),
-                    torchvision.transforms.Normalize((0.1307,), (0.3081,))
-                ])
-            ),
-            batch_size=self.batch_size,
-            shuffle=False
-        )
-
     def load_local_ids(self) -> List[str]:
-        # train_ids = [str(_id) for _id in self.train_loader.sampler]
-        # test_ids = [str(-1 - _id) for _id in self.test_loader.sampler]
-        # return train_ids + test_ids
-        return [str(i) for i in range(-500, 1000)]
+        train_ids = [str(i) for i in range(0, 20000)]
+        test_ids = [str(i) for i in range(100000, 105000)]
+        return train_ids + test_ids
 
     def split_dataset(self, id_intersection: Set[str]) -> Tuple[Set[str], Set[str]]:
-        ids = set(int(_id) for _id in id_intersection)
-        train_ids = set(str(_id) for _id in ids if _id >= 0)
-        test_ids = set(str(_id + 1) for _id in ids if _id < 0)
-        return train_ids, test_ids
+        ids = [int(_id) for _id in id_intersection]
+        ids.sort()
+        train_ids = ids[:10000]
+        test_ids = ids[10000:]
+
+        logger.info(f'Got {len(train_ids)} intersecting samples for training.')
+        logger.info(f'Got {len(test_ids)} intersecting samples for testing.')
+
+        return set(train_ids), set(test_ids)
 
     def make_feature_model(self) -> nn.Module:
         return ConvNet()
@@ -382,49 +331,50 @@ class DemoSecureHeteroHost(SecureHeteroNNHostScheduler):
 
     def iterate_train_feature(self,
                               feature_model: nn.Module,
-                              train_ids: List[str]) -> Tuple[torch.Tensor, torch.Tensor]:
-        # assert len(train_ids) == 60000, 'Some train samples lost.'
-        # for _data, _labels in self.train_loader:
-        #     _data = self._erase_right(_data)
-        #     yield feature_model(_data), _labels
-        quota = len(train_ids)
+                              train_ids: Set[str]) -> Tuple[torch.Tensor, torch.Tensor]:
+        train_dataset = torchvision.datasets.MNIST(
+            self.data_dir,
+            train=True,
+            download=True,
+            transform=torchvision.transforms.Compose([
+                torchvision.transforms.ToTensor(),
+                torchvision.transforms.Normalize((0.1307,), (0.3081,))
+            ])
+        )
+        train_ids: List = list(train_ids)
+        train_ids.sort(key=lambda x: md5(bytes(x + self.current_round)).digest())
+        train_dataset = Subset(train_dataset, train_ids)
+        self.train_loader = DataLoader(train_dataset,
+                                       batch_size=self.batch_size,
+                                       shuffle=False,
+                                       drop_last=True)
+
         for _data, _labels in self.train_loader:
-            if len(_data) < quota:
-                quota -= len(_data)
-                _data = self._erase_right(_data)
-                yield feature_model(_data), _labels
-            elif len(_data) == quota:
-                _data = self._erase_right(_data)
-                yield feature_model(_data), _labels
-                break
-            else:
-                _data, _labels = _data[:quota], _labels[:quota]
-                _data = self._erase_right(_data)
-                yield feature_model(_data), _labels
-                break
+            _data = self._erase_right(_data)
+            yield feature_model(_data), _labels
 
     def iterate_test_feature(self,
                              feature_model: nn.Module,
-                             test_ids: List[str]) -> Tuple[torch.Tensor, torch.Tensor]:
-        # assert len(test_ids) == 10000, 'Some test samples lost.'
-        # for _data, _labels in self.test_loader:
-        #     _data = self._erase_right(_data)
-        #     yield feature_model(_data), _labels
-        quota = len(test_ids)
+                             test_ids: Set[str]) -> Tuple[torch.Tensor, torch.Tensor]:
+        test_dataset = torchvision.datasets.MNIST(
+            self.data_dir,
+            train=False,
+            download=True,
+            transform=torchvision.transforms.Compose([
+                torchvision.transforms.ToTensor(),
+                torchvision.transforms.Normalize((0.1307,), (0.3081,))
+            ])
+        )
+        test_ids = [(i - 100000) for i in test_ids]
+        test_dataset = Subset(test_dataset, test_ids)
+        self.test_loader = DataLoader(test_dataset,
+                                      batch_size=self.batch_size,
+                                      shuffle=False,
+                                      drop_last=True)
+
         for _data, _labels in self.test_loader:
-            if len(_data) < quota:
-                quota -= len(_data)
-                _data = self._erase_right(_data)
-                yield feature_model(_data), _labels
-            elif len(_data) == quota:
-                _data = self._erase_right(_data)
-                yield feature_model(_data), _labels
-                break
-            else:
-                _data, _labels = _data[:quota], _labels[:quota]
-                _data = self._erase_right(_data)
-                yield feature_model(_data), _labels
-                break
+            _data = self._erase_right(_data)
+            yield feature_model(_data), _labels
 
     def make_infer_model(self) -> nn.Module:
         return InferModule()
@@ -470,26 +420,6 @@ class DemoSecureHeteroHost(SecureHeteroNNHostScheduler):
         self.tb_writer.add_scalar('test_results/accuracy', accuracy, self.current_round)
         self.tb_writer.add_scalar('test_results/correct_rate', correct_rate, self.current_round)
 
-    def validate_context(self):
-        super().validate_context()
-        assert self.train_loader and len(self.train_loader) > 0, 'failed to load train data'
-        logger.info(f'There are {len(self.train_loader.dataset)} samples for training.')
-        assert self.test_loader and len(self.test_loader) > 0, 'failed to load test data'
-        logger.info(f'There are {len(self.test_loader.dataset)} samples for testing.')
-
-    # replace data channel ports used for debuging
-    def _make_id_intersection(self) -> List[str]:
-        """Make PSI and get id intersection for training."""
-        local_ids = self.load_local_ids()
-        psi_scheduler = RSAPSIInitiatorScheduler(
-            task_id=self.task_id,
-            initiator_id=self.id,
-            ids=local_ids,
-            collaborator_ids=self._partners,
-            contractor=self.contractor
-        )
-        self._id_intersection = psi_scheduler.make_intersection()
-
 
 class DemoSecureHeteroCollaborator(SecureHeteroNNCollaboratorScheduler):
 
@@ -498,54 +428,30 @@ class DemoSecureHeteroCollaborator(SecureHeteroNNCollaboratorScheduler):
                  project_layer_lr: int,
                  batch_size: int,
                  data_dir: str,
-                 name: str = None,
                  schedule_timeout: int = 30,
                  is_feature_trainable: bool = True) -> None:
         super().__init__(feature_key=feature_key,
                          project_layer_lr=project_layer_lr,
-                         name=name,
                          schedule_timeout=schedule_timeout,
                          is_feature_trainable=is_feature_trainable)
         self.batch_size = batch_size
         self.data_dir = data_dir
 
-        self.train_loader = td.DataLoader(
-            torchvision.datasets.MNIST(
-                self.data_dir,
-                train=True,
-                download=True,
-                transform=torchvision.transforms.Compose([
-                    torchvision.transforms.ToTensor(),
-                    torchvision.transforms.Normalize((0.1307,), (0.3081,))
-                ])
-            ),
-            batch_size=self.batch_size,
-            shuffle=False
-        )
-        self.test_loader = td.DataLoader(
-            torchvision.datasets.MNIST(
-                self.data_dir,
-                train=False,
-                download=True,
-                transform=torchvision.transforms.Compose([
-                    torchvision.transforms.ToTensor(),
-                    torchvision.transforms.Normalize((0.1307,), (0.3081,))
-                ])
-            ),
-            batch_size=self.batch_size,
-            shuffle=False
-        )
-
     def load_local_ids(self) -> List[str]:
-        train_ids = [str(_id) for _id in self.train_loader.sampler]
-        test_ids = [str(-1 - _id) for _id in self.test_loader.sampler]
+        train_ids = [str(i) for i in range(10000, 30000)]
+        test_ids = [str(i) for i in range(103000, 107000)]
         return train_ids + test_ids
 
     def split_dataset(self, id_intersection: Set[str]) -> Tuple[Set[str], Set[str]]:
-        ids = set(int(_id) for _id in id_intersection)
-        train_ids = set(str(_id) for _id in ids if _id >= 0)
-        test_ids = set(str(_id + 1) for _id in ids if _id < 0)
-        return train_ids, test_ids
+        ids = [int(_id) for _id in id_intersection]
+        ids.sort()
+        train_ids = ids[:10000]
+        test_ids = ids[10000:]
+
+        logger.info(f'Got {len(train_ids)} intersecting samples for training.')
+        logger.info(f'Got {len(test_ids)} intersecting samples for testing.')
+
+        return set(train_ids), set(test_ids)
 
     def make_feature_model(self) -> nn.Module:
         return ConvNet()
@@ -558,68 +464,50 @@ class DemoSecureHeteroCollaborator(SecureHeteroNNCollaboratorScheduler):
 
     def iterate_train_feature(self,
                               feature_model: nn.Module,
-                              train_ids: List[str]) -> torch.Tensor:
-        # assert len(train_ids) == 60000, 'Some train samples lost.'
-        # for _data, _ in self.train_loader:
-        #     _data = self._erase_left(_data)
-        #     yield feature_model(_data)
-        quota = len(train_ids)
+                              train_ids: Set[str]) -> torch.Tensor:
+        train_dataset = torchvision.datasets.MNIST(
+            self.data_dir,
+            train=True,
+            download=True,
+            transform=torchvision.transforms.Compose([
+                torchvision.transforms.ToTensor(),
+                torchvision.transforms.Normalize((0.1307,), (0.3081,))
+            ])
+        )
+        train_ids: List = list(train_ids)
+        train_ids.sort(key=lambda x: md5(bytes(x + self.current_round)).digest())
+        train_dataset = Subset(train_dataset, train_ids)
+        self.train_loader = DataLoader(train_dataset,
+                                       batch_size=self.batch_size,
+                                       shuffle=False,
+                                       drop_last=True)
+
         for _data, _ in self.train_loader:
-            if len(_data) < quota:
-                quota -= len(_data)
-                _data = self._erase_left(_data)
-                yield feature_model(_data)
-            elif len(_data) == quota:
-                _data = self._erase_left(_data)
-                yield feature_model(_data)
-                break
-            else:
-                _data = _data[:quota]
-                _data = self._erase_left(_data)
-                yield feature_model(_data)
-                break
+            _data = self._erase_left(_data)
+            yield feature_model(_data)
 
     def iterate_test_feature(self,
                              feature_model: nn.Module,
-                             test_ids: List[str]) -> torch.Tensor:
-        # assert len(test_ids) == 10000, 'Some test samples lost.'
-        # for _data, _ in self.test_loader:
-        #     _data = self._erase_left(_data)
-        #     yield feature_model(_data)
-        quota = len(test_ids)
-        for _data, _ in self.test_loader:
-            if len(_data) < quota:
-                quota -= len(_data)
-                _data = self._erase_left(_data)
-                yield feature_model(_data)
-            elif len(_data) == quota:
-                _data = self._erase_left(_data)
-                yield feature_model(_data)
-                break
-            else:
-                _data = _data[:quota]
-                _data = self._erase_left(_data)
-                yield feature_model(_data)
-                break
-
-    def validate_context(self):
-        super().validate_context()
-        assert self.train_loader and len(self.train_loader) > 0, 'failed to load train data'
-        logger.info(f'There are {len(self.train_loader.dataset)} samples for training.')
-        assert self.test_loader and len(self.test_loader) > 0, 'failed to load test data'
-        logger.info(f'There are {len(self.test_loader.dataset)} samples for testing.')
-
-    # replace data channel ports used for debuging
-    def _make_id_intersection(self) -> List[str]:
-        """Make PSI and get id intersection for training."""
-        local_ids = self.load_local_ids()
-        psi_scheduler = RSAPSICollaboratorScheduler(
-            task_id=self.task_id,
-            collaborator_id=self.id,
-            ids=local_ids,
-            contractor=self.contractor
+                             test_ids: Set[str]) -> torch.Tensor:
+        test_dataset = torchvision.datasets.MNIST(
+            self.data_dir,
+            train=False,
+            download=True,
+            transform=torchvision.transforms.Compose([
+                torchvision.transforms.ToTensor(),
+                torchvision.transforms.Normalize((0.1307,), (0.3081,))
+            ])
         )
-        self._id_intersection = psi_scheduler.collaborate_intersection()
+        test_ids = [(i - 100000) for i in test_ids]
+        test_dataset = Subset(test_dataset, test_ids)
+        self.test_loader = DataLoader(test_dataset,
+                                      batch_size=self.batch_size,
+                                      shuffle=False,
+                                      drop_last=True)
+
+        for _data, _ in self.test_loader:
+            _data = self._erase_left(_data)
+            yield feature_model(_data)
 
 
 def get_task_id() -> str:
@@ -636,24 +524,22 @@ def get_host(mode: str = VANILLA) -> Union[DemoHeteroHost, DemoSecureHeteroHost]
 
     if mode == VANILLA:
         scheduler = DemoHeteroHost(feature_key='demo_host',
-                                   batch_size=1000,
+                                   batch_size=128,
                                    data_dir=_DATA_DIR,
-                                   name='demo_hetero_host',
                                    max_rounds=5,
                                    calculation_timeout=60,
                                    log_rounds=1)
     else:
         project_layer_config = [
-            ('demo_host', 10, 10),
-            ('demo_collaborator', 10, 10)
+            ('demo_host', 20, 20),
+            ('demo_collaborator', 20, 20)
         ]
         # Too big batch size could kill the server
         scheduler = DemoSecureHeteroHost(feature_key='demo_host',
                                          project_layer_config=project_layer_config,
                                          project_layer_lr=0.01,
-                                         batch_size=500,
+                                         batch_size=128,
                                          data_dir=_DATA_DIR,
-                                         name='demo_hetero_host',
                                          max_rounds=5,
                                          calculation_timeout=60,
                                          log_rounds=1)
@@ -677,17 +563,15 @@ def get_collaborator(mode: str = VANILLA) -> Union[DemoHeteroCollaborator,
 
     if mode == VANILLA:
         scheduler = DemoHeteroCollaborator(feature_key='demo_collaborator',
-                                           batch_size=1000,
-                                           data_dir=_DATA_DIR,
-                                           name='demo_hetero_collaborator')
+                                           batch_size=128,
+                                           data_dir=_DATA_DIR)
 
     else:
         # Too big batch size could kill the server
         scheduler = DemoSecureHeteroCollaborator(feature_key='demo_collaborator',
                                                  project_layer_lr=0.01,
-                                                 batch_size=500,
-                                                 data_dir=_DATA_DIR,
-                                                 name='demo_hetero_collaborator')
+                                                 batch_size=128,
+                                                 data_dir=_DATA_DIR)
 
     with open(pickle_file, 'w+b') as pf:
         pickle.dump(scheduler, pf)
