@@ -2,10 +2,12 @@
 
 
 import secrets
+from abc import ABC
 from dataclasses import dataclass
 from typing import List
 
-from ..contractor import (ApplySendingDataEvent, ContractEvent,
+from ..contractor import (ApplySharedFileSendingDataEvent,
+                          AutoTaskMessageContractor, ContractEvent,
                           TaskMessageContractor, TaskMessageEventFactory)
 
 
@@ -92,60 +94,6 @@ class SyncStateResponseEvent(ContractEvent):
         return SyncStateResponseEvent(round=round, peer_id=peer_id)
 
 
-@dataclass
-class StartAggregatorElectionEvent(ContractEvent):
-    """An event of starting to elect a new aggregator for next round of training."""
-
-    TYPE = 'start_aggregator_election'
-
-    round: int
-
-    @classmethod
-    def contract_to_event(cls, contract: dict) -> 'StartAggregatorElectionEvent':
-        event_type = contract.get('type')
-        round = contract.get('round')
-        assert event_type == cls.TYPE, f'合约类型错误: {event_type}'
-        assert isinstance(round, int) and round > 0, f'invalid round: {round}'
-        return StartAggregatorElectionEvent(round=round)
-
-
-@dataclass
-class BenchmarkDoneEvent(ContractEvent):
-    """An event of broadcasting completion of the benchmark task."""
-
-    TYPE = 'benchmark_done'
-
-    round: int
-    peer_id: str
-
-    @classmethod
-    def contract_to_event(cls, contract: dict) -> 'BenchmarkDoneEvent':
-        event_type = contract.get('type')
-        round = contract.get('round')
-        peer_id = contract.get('peer_id')
-        assert event_type == cls.TYPE, f'合约类型错误: {event_type}'
-        assert isinstance(round, int) and round > 0, f'invalid round: {round}'
-        assert peer_id and isinstance(peer_id, str), f'invalid peer_id: {peer_id}'
-        return BenchmarkDoneEvent(round=round, peer_id=peer_id)
-
-
-@dataclass
-class CloseAggregatorElectionEvent(ContractEvent):
-    """An event of starting to elect a new aggregator for next round of training."""
-
-    TYPE = 'close_aggregator_election'
-
-    round: int
-
-    @classmethod
-    def contract_to_event(cls, contract: dict) -> 'CloseAggregatorElectionEvent':
-        event_type = contract.get('type')
-        round = contract.get('round')
-        assert event_type == cls.TYPE, f'合约类型错误: {event_type}'
-        assert isinstance(round, int) and round > 0, f'invalid round: {round}'
-        return CloseAggregatorElectionEvent(round=round)
-
-
 @dataclass
 class StartRoundEvent(ContractEvent):
     """An event of starting a new round of training."""
@@ -221,10 +169,23 @@ class FinishTaskEvent(ContractEvent):
         return FinishTaskEvent()
 
 
+@dataclass
+class ResetRoundEvent(ContractEvent):
+    """An event of resetting context of current training round."""
+
+    TYPE = 'reset_round'
+
+    @classmethod
+    def contract_to_event(cls, contract: dict) -> 'ResetRoundEvent':
+        event_type = contract.get('type')
+        assert event_type == cls.TYPE, f'合约类型错误: {event_type}'
+        return ResetRoundEvent()
+
+
 # define some more friendly alias
-SynchronizeAggregatorContextEvent = ApplySendingDataEvent
-DistributeParametersEvent = ApplySendingDataEvent
-UploadTrainingResultsEvent = ApplySendingDataEvent
+SynchronizeAggregatorContextEvent = ApplySharedFileSendingDataEvent
+DistributeParametersEvent = ApplySharedFileSendingDataEvent
+UploadTrainingResultsEvent = ApplySharedFileSendingDataEvent
 
 
 class FedAvgEventFactory(TaskMessageEventFactory):
@@ -234,22 +195,16 @@ class FedAvgEventFactory(TaskMessageEventFactory):
         CheckinResponseEvent.TYPE: CheckinResponseEvent,
         SyncStateEvent.TYPE: SyncStateEvent,
         SyncStateResponseEvent.TYPE: SyncStateResponseEvent,
-        StartAggregatorElectionEvent.TYPE: StartAggregatorElectionEvent,
-        BenchmarkDoneEvent.TYPE: BenchmarkDoneEvent,
-        CloseAggregatorElectionEvent.TYPE: CloseAggregatorElectionEvent,
         StartRoundEvent.TYPE: StartRoundEvent,
         ReadyForAggregationEvent.TYPE: ReadyForAggregationEvent,
         CloseRoundEvent.TYPE: CloseRoundEvent,
         FinishTaskEvent.TYPE: FinishTaskEvent,
+        ResetRoundEvent.TYPE: ResetRoundEvent,
         **TaskMessageEventFactory._CLASS_MAP
     }
 
 
-class FedAvgContractor(TaskMessageContractor):
-
-    def __init__(self, task_id: str):
-        super().__init__(task_id=task_id)
-        self._event_factory = FedAvgEventFactory
+class FedAvgContractorMixin(ABC):
 
     def checkin(self, peer_id: str) -> str:
         """Checkin to the task.
@@ -262,11 +217,11 @@ class FedAvgContractor(TaskMessageContractor):
         self._new_contract(targets=self.EVERYONE, event=event)
         return nonce
 
-    def respond_checkin(self,
-                        round: int,
-                        aggregator: str,
-                        nonce: str,
-                        requester_id: str):
+    def respond_check_in(self,
+                         round: int,
+                         aggregator: str,
+                         nonce: str,
+                         requester_id: str):
         """Respond checkin event."""
         event = CheckinResponseEvent(round=round, aggregator=aggregator, nonce=nonce)
         self._new_contract(targets=[requester_id], event=event)
@@ -284,21 +239,6 @@ class FedAvgContractor(TaskMessageContractor):
         event = SyncStateResponseEvent(round=round, peer_id=peer_id)
         self._new_contract(targets=[aggregator], event=event)
 
-    def start_aggregator_election(self, round: int):
-        """Start to elect a new aggregator for next round."""
-        event = StartAggregatorElectionEvent(round=round)
-        self._new_contract(targets=self.EVERYONE, event=event)
-
-    def report_benchmark_done(self, round: int, peer_id: str):
-        """Broadcast completion of the benchmark."""
-        event = BenchmarkDoneEvent(round=round, peer_id=peer_id)
-        self._new_contract(targets=self.EVERYONE, event=event)
-
-    def close_aggregator_election(self, round: int):
-        """Close to elect a new aggregator for next round."""
-        event = CloseAggregatorElectionEvent(round=round)
-        self._new_contract(targets=self.EVERYONE, event=event)
-
     def start_round(self,
                     calculators: List[str],
                     round: int,
@@ -323,3 +263,22 @@ class FedAvgContractor(TaskMessageContractor):
         """Finish the specified task."""
         event = FinishTaskEvent()
         self._new_contract(targets=self.EVERYONE, event=event)
+
+    def reset_round(self):
+        """Reset current training round."""
+        event = ResetRoundEvent()
+        self._new_contract(targets=self.EVERYONE, event=event)
+
+
+class FedAvgContractor(TaskMessageContractor, FedAvgContractorMixin):
+
+    def __init__(self, task_id: str):
+        super().__init__(task_id=task_id)
+        self._event_factory = FedAvgEventFactory
+
+
+class AutoFedAvgContractor(AutoTaskMessageContractor, FedAvgContractorMixin):
+
+    def __init__(self, task_id: str):
+        super().__init__(task_id=task_id)
+        self._event_factory = FedAvgEventFactory
