@@ -5,12 +5,10 @@ Reference: https://arxiv.org/abs/1710.06963
 
 
 import io
-import random
-import sys
 from abc import ABCMeta, abstractmethod
 from copy import deepcopy
 from math import sqrt
-from typing import Dict, Tuple, final
+from typing import Dict, final
 
 import torch
 
@@ -26,26 +24,19 @@ class DPFedAvgScheduler(FedAvgScheduler, metaclass=ABCMeta):
     """Implementation of DPFedAvg."""
 
     def __init__(self,
-                 min_clients: int,
                  w_cap: int,
                  q: float,
                  S: float,
                  z: float,
-                 name: str = None,
                  max_rounds: int = 0,
                  merge_epochs: int = 1,
                  calculation_timeout: int = 300,
-                 perf_bench_timeout: int = 30,
                  schedule_timeout: int = 30,
-                 data_channel_timeout: Tuple[int, int] = (30, 60),
                  log_rounds: int = 0,
-                 is_centralized: bool = True,
                  involve_aggregator: bool = False):
-        r"""Init.
+        """Init.
 
         :args
-            :min_clients
-                Minimal number of calculators for each round.
             :w_hat
                 Per-user example cap. (See the paper.)
             :q
@@ -59,36 +50,18 @@ class DPFedAvgScheduler(FedAvgScheduler, metaclass=ABCMeta):
             :calculation_timeout
                 Seconds to timeout for calculation in a round. Takeing off timeout
                 by setting its value to 0.
-            :perf_bench_timeout
-                Seconds to timeout for performing performance benchmark. Takeing off timeout
-                by setting its value to 0.
             :schedule_timeout
                 Seconds to timeout for process scheduling. Takeing off timeout
                 by setting its value to 0.
-            :data_channel_timeout
-                NOTE: This is a temporary configuration. May be removed in next version.
-                Do not design your process depending on it.\n
-                A pair of timeout configuration. The first is seconds to timeout for
-                connecting, and the second is seconds to timeout for transmitting data.
-                Takeing off either one by setting its value to 0.
             :log_rounds
                 The number of rounds to run testing and log the result. Skip it
                 by setting its value to 0.
-            :is_centralized
-                If specify centralized, the aggregator will always be the initiator of the
-                task, otherwize a new aggregator is elected for each round during training.
         """
-        super().__init__(min_clients=min_clients,
-                         max_clients=sys.maxsize,
-                         name=name,
-                         max_rounds=max_rounds,
+        super().__init__(max_rounds=max_rounds,
                          merge_epochs=merge_epochs,
                          calculation_timeout=calculation_timeout,
-                         perf_bench_timeout=perf_bench_timeout,
                          schedule_timeout=schedule_timeout,
-                         data_channel_timeout=data_channel_timeout,
                          log_rounds=log_rounds,
-                         is_centralized=is_centralized,
                          involve_aggregator=involve_aggregator)
         self.w_cap = w_cap
         self.q = float(q)
@@ -132,28 +105,21 @@ class DPFedAvgScheduler(FedAvgScheduler, metaclass=ABCMeta):
                 A list of data in a batch from training data loader.
         """
 
-    def _select_calculators(self):
-        if len(self._calculators) < self.min_clients:
-            raise AggregationError(f'too few calculators: {len(self._calculators)}')
-        self._calculators = [_peer for _peer in self._calculators if random.random() < self.q]
-        if len(self._calculators) < self.min_clients:
-            raise AggregationError(f'too few calculators: {len(self._calculators)}')
-
     def _handle_check_in(self, _event: DPCheckinEvent):
-        if _event.peer_id not in self._participants:
-            self._participants.append(_event.peer_id)
+        if _event.peer_id not in self.gethered:
+            self.gethered.append(_event.peer_id)
             self._w_ks[_event.peer_id] = self._calc_w_k(_event.n_k)
             self.push_log(f'Welcome a new participant ID: {_event.peer_id}.')
-            self.push_log(f'There are {len(self._participants)} participants now.')
-        self.contractor.respond_checkin(round=self._round,
-                                        aggregator=self.id,
-                                        nonce=_event.nonce,
-                                        requester_id=_event.peer_id)
+            self.push_log(f'There are {len(self.gethered) + 1} participants now.')
+        self.contractor.respond_check_in(round=self.current_round,
+                                         aggregator=self.id,
+                                         nonce=_event.nonce,
+                                         requester_id=_event.peer_id)
 
     def _check_in_task(self) -> bool:
         """Try to check in the task."""
         nonce = self.contractor.checkin(peer_id=self.id, n_k=len(self.train_loader.dataset))
-        return self._wait_for_checkin_response(nonce=nonce, timeout=self.schedule_timeout)
+        return self._wait_for_check_in_response(nonce=nonce, timeout=self.schedule_timeout)
 
     def _execute_training(self):
         self.model.train()
@@ -185,11 +151,11 @@ class DPFedAvgScheduler(FedAvgScheduler, metaclass=ABCMeta):
 
     def _process_aggregation(self):
         self._switch_status(self._WAIT_FOR_AGGR)
-        self.contractor.notify_ready_for_aggregation(round=self._round)
+        self.contractor.notify_ready_for_aggregation(round=self.current_round)
         self.push_log('Now waiting for executing calculation ...')
 
         state_dicts = self._wait_for_calculation()
-        if len(state_dicts) < self.min_clients:
+        if len(state_dicts) < len(self.gethered):
             self.push_log('Task failed because of too few calculation results gathered.')
             raise AggregationError(f'Too few results gathered: {len(state_dicts)} copies.')
         self.push_log(f'Received {len(state_dicts)} copies of calculation results.')
@@ -233,7 +199,7 @@ class DPFedAvgScheduler(FedAvgScheduler, metaclass=ABCMeta):
 
     def _wait_for_calculation(self) -> Dict[str, Dict[str, torch.Tensor]]:
         """Wait for every calculator finish its task or timeout."""
-        if self.id in self._calculators:
+        if self.involve_aggregator:
             self._execute_training()
             self.push_log(f'The aggregator ID: {self.id} obtained its calculation results.')
             state_dicts = {self.id: self.state_dict()}
@@ -243,17 +209,15 @@ class DPFedAvgScheduler(FedAvgScheduler, metaclass=ABCMeta):
         self.push_log('Waiting for training results ...')
         for _event in self.contractor.contract_events(timeout=self.calculation_timeout):
             if isinstance(_event, UploadTrainingResultsEvent):
-                if _event.target != self.id:
+                if self.id not in _event.target:
                     continue
-                training_result = self.data_channel.receive_stream(_event)
+                training_result = self.data_channel.receive_stream(apply_event=_event,
+                                                                   receiver=self.id)
                 buffer = io.BytesIO(training_result)
                 _new_state_dict = torch.load(buffer)
                 state_dicts[_event.source] = _new_state_dict
                 self.push_log(f'Received calculation results from ID: {_event.source}')
-                if len(state_dicts) >= len(self._calculators):
+                if len(state_dicts) == len(self.gethered) + int(self.involve_aggregator):
                     return state_dicts
 
-            elif isinstance(_event, DPCheckinEvent):
-                self._handle_check_in(_event)
-
         return state_dicts
