@@ -272,7 +272,7 @@ class FedAvgScheduler(Scheduler, metaclass=ABCMeta):
         self.is_initiator = is_initiator
         self.is_aggregator = self.is_initiator
 
-        self.contractor = FedAvgContractor(task_id=task_id)
+        self.contractor = self._init_contractor()
         self.data_channel = SharedFileDataChannel(self.contractor)
         self.model
         self.optimizer
@@ -284,6 +284,9 @@ class FedAvgScheduler(Scheduler, metaclass=ABCMeta):
         self.push_log(message='Begin to validate local context.')
         self.validate_context()
 
+    def _init_contractor(self):
+        return FedAvgContractor(task_id=self.task_id)
+
     def _recover_progress(self):
         """Try to recover and continue from last running."""
         if not os.path.isfile(self._context_file):
